# -*- coding: utf-8 -*-
"""
Fantail
"""
from __future__ import print_function

from fantail.core import Fantail
from fantail.stack import Fanstack

from fantail.util import dict_loader, yaml_string_loader, \
    yaml_file_loader, load, yaml_file_save, dir_loader, \
    package_loader
